<?php

namespace HCM\BTL\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;

class CandidatosFaseTemplateController extends Crud {

    protected $dataSourceName = 'des_participafase';
    protected $dataSourceName2 = 'des_participafase_buscaDelete';
    protected $dataSourceName3 = 'setStatus_Des_participaprocesso';
    protected $parameterBagColumns = array('NRORG' , 'NRPARCNEGOCIO', 'NRORDEM');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag){
        $this->parameterBag = $parameterBag;
        parent::__construct($dataSourceManager);
    }

     public function buscarDados (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName2);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }

    public function salvarCandidatosFaseTemplate (DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                if(array_key_exists('DSSTATUS', $row)){
                    $aux = $this->dataSourceName;
                    $this->dataSourceName = $this->dataSourceName3;
                    $row['oldIsNew'] = $row['__is_new'];
                    $row['__is_new'] = false;
                    $row['CDSTATUS'] = 05;
                    $row['NRPARCNEGOCIOCAND'] = $row['CANDIDATOS'][0]['NRPARCNEGOCIOCAND'];
                    parent::save($request, $response);
                    $row['__is_new'] = $row['oldIsNew'];
                    $this->dataSourceName = $aux;
                }
                foreach ($row["CANDIDATOS"] as $rows) {
                    if(isset($rows[0])){
                        foreach ($rows as $dado) {
                        $dado['__is_new'] = $row['__is_new'];
                            $dado = self::salvar($dado, $row);
                        }
                        $requestSave = new DTO\Request\DataSet ((new \Zeedhi\Framework\DataSource\DataSet('des_participafase', array($dado))), $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                        parent::save($requestSave, $response);
                    } else {
                        $rows['__is_new'] = $row['__is_new'];
                        $rows = self::salvar($rows, $row);
                        $requestSave = new DTO\Request\DataSet ((new \Zeedhi\Framework\DataSource\DataSet('des_participafase', array($rows))), $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                        parent::save($requestSave, $response);
                    }
                    
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function salvar ($rows, $row){

        if(array_key_exists('NRDURACAO', $rows)){
            $rows['NRDURACAO'] = DateUtil::getHoraFloatDeString($rows['NRDURACAO']);
        }
        if(array_key_exists('__is_new', $row)){
            $rows['__is_new'] = $row['__is_new'];
        }

        if(array_key_exists('NRPROCESSO', $row)){
            $rows['NRPROCESSO'] = $row['NRPROCESSO'];
        }

        if(array_key_exists('NRTEMPLATE', $row)){
            $rows['NRTEMPLATE'] = $row['NRTEMPLATE'];
        }

        if(array_key_exists('NRFASE', $row)){
            $rows['NRFASE'] = $row['NRFASE'];
        }

        if(array_key_exists('CDSTATUS', $rows)){
            $rows['IDSTATUS'] = $rows['CDSTATUS'];
        }

        if(array_key_exists('NRPARCTICIPAFASE', $rows)){
            if($rows['NRPARCTICIPAFASE'] === '' ){
                $rows['NRPARCTICIPAFASE'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'DES_PARTICIPAFASE', 1, 12, true);
            }
        } else {
            $rows['NRPARCTICIPAFASE'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'DES_PARTICIPAFASE', 1, 12, true);
        }

        if(!array_key_exists('DTINCLUSAO', $rows)){
          $rows['DTINCLUSAO'] = date('d/m/Y');
        } else {
            $explode = explode(" ", $rows['DTINCLUSAO']);
            if(isset($explode[1])){
                $rows['DTINCLUSAO'] = $explode[0];
            }
        }

        if(!array_key_exists('DTULTATU', $rows)){
          $rows['DTULTATU'] = date('d/m/Y');
        } else {
            $explode = explode(" ", $rows['DTULTATU']);
            if(isset($explode[1])){
                $rows['DTULTATU'] = $explode[0];
            }
        }

        if(array_key_exists('NRPERCENTUAL', $rows)){
            $rows['NRPERCENTUAL'] = str_replace("%", "", $rows['NRPERCENTUAL']);
        }

        if(!array_key_exists('NRORGINCLUSAO', $rows)){
            $rows['NRORGINCLUSAO'] = $row['NRORG'];
        }

        if(!array_key_exists('NRORGULTATU', $rows)){
            $rows['NRORGULTATU'] = $row['NRORG'];
        }

        if(!array_key_exists('CDOPERINCLUSAO', $rows)){
            $rows['CDOPERINCLUSAO'] = $row['CDOPERADOR_LOG'];
        }

        if(!array_key_exists('CDOPERULTATU', $rows)){
            $rows['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
        }

        if(!array_key_exists('NRDURACAO', $rows)){
            $rows['NRDURACAO'] = null;
        }

        if(!array_key_exists('NRPESO', $rows)){
            $rows['NRPESO'] = null;
        }

        if(!array_key_exists('DTINICIO', $rows)){
            $rows['DTINICIO'] = null;
        }

        if(!array_key_exists('DTFIM', $rows)){
            $rows['DTFIM'] = null;
        }

        if(!array_key_exists('NRPONTUACAO', $rows)){
            $rows['NRPONTUACAO'] = null;
        } else {
            $rows['NRPONTUACAO'] = str_replace(",", ".", $rows['NRPONTUACAO']);
        }

        if(!array_key_exists('NRAVALIACAO', $rows)){
            $rows['NRAVALIACAO'] = null;
        }

        if(!array_key_exists('DSLOCALFASE', $rows)){
            $rows['DSLOCALFASE'] = null;
        }

        if(!array_key_exists('IDOBRIGATORIO', $rows)){
            $rows['IDOBRIGATORIO'] = null;
        }

        if(!array_key_exists('NRFASERELACIONADA', $rows))
            $rows['NRFASERELACIONADA'] = null;

        if(!array_key_exists('DTINICIOFASE', $rows)){
            $rows['DTINICIOFASE'] = null;
        }

        if(!array_key_exists('DTFIMFASE', $rows)){
            $rows['DTFIMFASE'] = null;
        }

        if(!array_key_exists('CDTIPORESU', $rows)){
            $rows['CDTIPORESU'] = null;
        }

        if(!array_key_exists('NRPERCENTUAL', $rows)){
            $rows['NRPERCENTUAL'] = null;
        }
        return $rows;
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach($request->getDataSet()->getRows() as $row) {
                foreach($row as $rows){
                    $requestDelete = new DTO\Request\DataSet ((new \Zeedhi\Framework\DataSource\DataSet('des_participafase', array($rows))), $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                    parent::delete($requestDelete, $response);
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}