<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\FPA\Services\AssociacaoContabilService;

class AssociacaoContabilController extends Crud {
 
    protected $dataSourceName     = 'fpa_eventoctbl_get';
    protected $dataSourceNameSet  = 'fpa_eventoctbl';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_DTMESCOMPETENC', 'P_NRORG_PADRAO');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, AssociacaoContabilService $associacaoContabilService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->associacaoContabilService = $associacaoContabilService;
        parent::__construct($dataSourceManager);
    }
 
    public function getAssociacaoContabil(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria(); 
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    } 
    
    public function setAssociacaoContabil(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $existe = false;
            foreach ($request->getDataSet()->getRows() as &$row) {
                if(!array_key_exists('CDCONTCTBLDEB', $row)){
                    $row['CDCONTCTBLDEB'] = null;
                }
                if(!array_key_exists('CDHISTPADRDEB', $row)){
                    $row['CDHISTPADRDEB'] = null;
                }
                if(!array_key_exists('CDCONTCTBLCRE', $row)){
                    $row['CDCONTCTBLCRE'] = null;
                }
                if(!array_key_exists('CDHISTPADRCRE', $row)){
                    $row['CDHISTPADRCRE'] = null;
                }
                if(!array_key_exists('NRTIPOMOVIMENT', $row)){
                    $row['NRTIPOMOVIMENT'] = null;
                }
                if(!array_key_exists('CDDESPADMFAT', $row)){
                    $row['CDDESPADMFAT'] = null;
                }
                if(!array_key_exists('NRTPEVENTOCTBL', $row)){
                    $row['NRTPEVENTOCTBL'] = null;
                }
                if ($row['__is_new'] ) {
                    $row['NREVENTOCTBL'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_EVENTOCTBL', 1, 12, true); 
                }
            }
            $this->dataSourceName =  $this->dataSourceNameSet;
            parent::save($request, $response);
                
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteAssociacaoContabil(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = 'fpa_eventoctbl';
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    public function cadAutoAssocCtbl(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                foreach($row['NREVENTOM'] as $chaveEvento => $valorEvento){
                    if(!empty($row['NRESTRUTURAM'])){
                        foreach($row['NRESTRUTURAM'] as $chaveEstrutura => $valorEstrutura){
                            $this->associacaoContabilService->cadAutoAssocCtbl($row, $chaveEvento, $chaveEstrutura);
                        }
                    }else
                        $this->associacaoContabilService->cadAutoAssocCtbl($row, $chaveEvento);
                }           
            }
            $response->addNotification(new DTO\Response\Notification('cadAutoAssocCtbl'), DTO\Response\Notification::TYPE_SUCCESS);

        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }    
    
    public function baixarPlanilhaTemplate(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $dataset = $request->getDataSet()->getRows();
            $row = $dataset[0];
            
            $file = $this->associacaoContabilService->baixarPlanilhaTemplate($row);
            if($file !== null){
                $retorno = array('NMARQUIVO' => $file);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $retorno));
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function importarPlanilha (DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0);
        
        try {
            $dataset = $request->getDataSet()->getRows();
            
            $row = $dataset[0];
            
            $mensagens = $this->associacaoContabilService->importarPlanilha($row);
            
            if(empty($mensagens)){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida com sucesso.", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            } else {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida. Existem erros na importação", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msg', $mensagens));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }

} 