<?php

namespace HCM\FPA\ObrigacaoLegal\CNAB\Santander\registro;

use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\MsgCalculo;
use HCM\FPA\Registro\RegistroCNAB;
use HCM\FPA\ObrigacaoLegal\CNAB\Santander\CNABSantander;

class Registro3BCNABSantander extends RegistroCNAB{

    const SEGMENTO_B = 'B';
    const CREDITO_HISTORICO_CREDITO_PADRAO = '2007';

    private $_pessoa;
    private $_vinculo;

    public function geraRegistro3B(){
        $this->_geraRegistro();
    }

    private function _geraRegistro(){
        // Bloco 1 ao 3 eh gerado no pai.
        parent::geraRegistro(self::REGISTRO_3, CNABSantander::CODIGO_BANCO_SANTANDER, self::LOTE_SERVICO);

        // Bloco 4 | 009-013 | Numero Sequencial do Registro no Lote.
        // identifica a sequencia de registros encaminhados no lote. Deve ser inicializado sempre em ‘00001’, em cada novo lote.
        // Definido apos a geracao de todos os registros do arquivo (a sequencia eh modificada no PercorreCaged).
        $this->atribuiDado( parent::CARACTER_ZERO, 5, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 5 | 014 | Codigo Segmento do Registro Detalhe.
        $this->atribuiDado(self::SEGMENTO_B, 1);

        // Bloco 6 | 015-017 | Filler.
        $this->atribuiDado(parent::CARACTER_BRANCO, 3, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 7 | 018 | Tipo de Inscricao do Favorecido. 1 = CPF / 2 = CNPJ.
        $this->atribuiDado($this->_pessoa->nrTpInscricao, 1);

        // Bloco 8 | 019-032 | Numero Inscricao do Favorecido (Obrigatorio). Pessoa Juridica pertencera a outro processo.
        $this->atribuiDado($this->_pessoa->historico->getNrcpfpessoa(), 14, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 9 | 033-062 | Logradouro do Favorecido (Opcional).
        $this->atribuiDado($this->_pessoa->endereco, 30, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 10 | 063-067 | Numero do Local do Favorecido (Opcional).
        $this->atribuiDado($this->_pessoa->numero, 5, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 11 | 068-082 | Complemento do Local Favorecido (Opcional).
        $this->atribuiDado($this->_pessoa->complemento, 15, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 12 | 083-097 | Bairro do Favorecido (Opcional).
        $this->atribuiDado($this->_pessoa->bairro, 15, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 13 | 098-117 | Cidade do Favorecido (Opcional).
        $this->atribuiDado($this->_pessoa->municipio, 20, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 14 | 118-125 | CEP do Favorecido (Opcional).
        $this->atribuiDado($this->_pessoa->cep, 8, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 15 | 126-127 | Estado do Favorecido (Opcional).
        $this->atribuiDado($this->_pessoa->estado, 2, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 16 | 128-135 | Data de Vencimento (DDMMAAAA). Data do pagamento.
        $this->atribuiDado($this->_vinculo->dataPagamento, 8, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 17 | 136-150 | Valor do Documento (Opcional). Salario liquido.
        $this->atribuiDado($this->_vinculo->valorPagamento, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 18 | 151-165 | Valor do Abatimento (Opcional). Nao informado.
        $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 19 | 166-180 | Valor do Desconto (Opcional). Nao informado.
        $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 20 | 181-195 | Valor da Mora (Opcional). Nao informado.
        $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 21 | 196-210 | Valor da Multa (Opcional). Nao informado.
        $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 22 | 211-214 | Horario de Envio de TED. Passando Zero.
        $this->atribuiDado(parent::CARACTER_ZERO, 4, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 23 | 215-225 | Filler.
        $this->atribuiDado(parent::CARACTER_BRANCO, 11, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 24 | 226-230 | Codigo Historico para Credito.
        // Codigos disponiveis no banco, que poderao ser utilizados para pagamentos atraves de Credito em Conta
        // Corrente ou Credito em Conta Poupança. OBS: O historico padrao utilizado pelo banco eh o 2007 – Credito Salario.
        // Para que outro historico possa ser utilizado, deve ser solicitada ao banco a atualização de
        // parametro proprio de seu cadastro no sistema.
        // 0013 Dividendos
        // 2029 Pagto de 13º Salário
        // 0021 Líquido Vencimento
        // 2030 Pagto de 14º Salário
        // 0087 Líquido Férias
        // 2031 Pagto Abono
        // 1060 Aviso Lançamento
        // 2032 Pagto Benefícios
        // 2001 Adiantamento Despesas
        // 2033 Pagto Bônus
        // 2002 Adiantamento Salarial
        // 2034 Pagto Gratificação
        // 2006 Crédito Cooperativas
        // 2035 Pagto Prêmio
        // 2007 Crédito Salário (eh o padrao do banco)
        // 2041 Participação nos Lucros
        $this->atribuiDado(self::CREDITO_HISTORICO_CREDITO_PADRAO, 5, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 26 | 231-240 | Ocorrencias para o Retorno.
        // Codigo adotado para identificar as ocorrencias detectadas no processamento.
        // Pode-se informar ate 5 ocorrencias simultaneamente, cada uma delas codificada com dois
        // digitos, conforme relacao das paginas 32 e 33.
        $this->atribuiDado(parent::CARACTER_BRANCO, 10, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
    }
    
    public function setPessoa($pessoa)
    {
        $this->_pessoa = $pessoa;
    }

    public function setVinculo($vinculo)
    {
        $this->_vinculo = $vinculo;
    }
}