<?php

namespace HCM\FPA\Services;

use Doctrine\ORM\EntityManager;
use Geral\Util\NovoCodigo;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\ManipulacaoExcel;
use HCM\Util\Repositories;
use HCM\FPA\Services\ExportacaoExcelService;
use HCM\FPA\Services\ContaContabilService;
use HCM\FPA\Entity\FpaeventoctblEntity;
use HCM\FPA\Repository\AssociacaoContabilRepository;
use HCM\Geral\Services\HistoricoPadraoService;
use Teknisa\Model\Gestaopessoas\Entities\FpaEventoctbl;
use Teknisa\Model\Gestaopessoas\Entities\fpaTpEventoCtbl;
use Teknisa\Model\Gestaooperacional\Entities\Despadmfatur;
use Zeedhi\Framework\HTTP\Kernel;

define('APP_PATH', dirname(__FILE__) . "/");

class AssociacaoContabilService {

    public function __construct(EntityManager $entityManager, Kernel $httpKernel, HCMParameters $hcmParameters, ExportacaoExcelService $exportacaoExcelService, AssociacaoContabilRepository $associacaoContabilRepository, HistoricoPadraoService $historicoPadraoService, ContaContabilService $contaContabilService) {
        $this->entityManager                = $entityManager;
        $this->hcmParameters                = $hcmParameters;
        $this->exportacaoExcelService       = $exportacaoExcelService;
        $this->httpKernel                   = $httpKernel;
        $this->mensagens                    = array();
        $this->associacaoContabilRepository = $associacaoContabilRepository;
        $this->_evento_contabil             = new FpaeventoctblEntity($entityManager, $hcmParameters);
        $this->fpaEventoCtbl                = Repositories::FPA_EVENTOCTBL;
        $this->fpaTpEventoCtbl              = Repositories::FPA_TIPOMOVIMENT;
        $this->histPadr                     = Repositories::HISTORICOPADRAO;
        $this->historicoPadraoService       = $historicoPadraoService;
        $this->contaContabilService         = $contaContabilService;
    }
 
    public function cadAutoAssocCtbl($row, $chaveEvento, $chaveEstrutura = NULL) {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $this->validaFields($row, $chaveEvento, $chaveEstrutura);

        $codigo = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_EVENTOCTBL', 1, 12, true);
        $obj = new FpaEventoctbl;
        $obj->setNrorg($this->hcmParameters->getNrorgTrabalho());
        $obj->setDtinclusao(DateUtil::getDataAtual());
        $obj->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $obj->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $obj->setDtultatu(DateUtil::getDataAtual());
        $obj->setCdoperultatu($this->hcmParameters->getCdoperador());
        $obj->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $obj->setDtinivigencia(DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true));
        $obj->setNreventom($row['NREVENTOM_CONT']);
        $obj->setNrestruturam($row['NRESTRUTURAM_CONT']);
        $obj->setCdcontctblcre($row['CDCONTCTBLCRE']);
        $obj->setCdcontctbldeb($row['CDCONTCTBLDEB']);
        $obj->setCdhistpadrcre($row['CDHISTPADRCRE']);
        $obj->setCdhistpadrdeb($row['CDHISTPADRDEB']);
        $obj->setCddespadmfat($row['CDDESPADMFAT']);
        $obj->setNrtipomoviment($row['NRTIPOMOVIMENT']);
        $obj->setNrtpeventoctbl($row['NRTPEVENTOCTBL']);
        $obj->setNreventoctbl($codigo);
        $this->entityManager->persist($obj);
        $this->entityManager->flush($obj);        
    }

    public function validaFields(&$row, $chaveEvento, $chaveEstrutura){
        $row['NREVENTOM_CONT'] = empty($row['NREVENTOM']) || !isset($row['NREVENTOM']) ? NULL : $row['NREVENTOM'][$chaveEvento];
        $row['NRESTRUTURAM_CONT'] = empty($row['NRESTRUTURAM']) || !isset($row['NRESTRUTURAM']) ? NULL : $row['NRESTRUTURAM'][$chaveEstrutura];
        $row['CDCONTCTBLCRE'] = empty($row['CDCONTCTBLCRE']) || !isset($row['CDCONTCTBLCRE']) ? NULL : $row['CDCONTCTBLCRE'];
        $row['CDCONTCTBLDEB'] = empty($row['CDCONTCTBLDEB']) || !isset($row['CDCONTCTBLDEB']) ? NULL : $row['CDCONTCTBLDEB'];
        $row['CDHISTPADRCRE'] = empty($row['CDHISTPADRCRE']) || !isset($row['CDHISTPADRCRE']) ? NULL : $row['CDHISTPADRCRE'];
        $row['CDHISTPADRDEB'] = empty($row['CDHISTPADRDEB']) || !isset($row['CDHISTPADRDEB']) ? NULL : $row['CDHISTPADRDEB'];
        $row['CDDESPADMFAT'] = empty($row['CDDESPADMFAT']) || !isset($row['CDDESPADMFAT']) ? NULL : $row['CDDESPADMFAT'];
        $row['NRTIPOMOVIMENT'] = empty($row['NRTIPOMOVIMENT']) || !isset($row['NRTIPOMOVIMENT']) ? NULL : $row['NRTIPOMOVIMENT'];    
        $row['NRTPEVENTOCTBL'] = empty($row['NRTPEVENTOCTBL']) || !isset($row['NRTPEVENTOCTBL']) ? NULL : $row['NRTPEVENTOCTBL'];
    }
    
    public function verificaExistenciaRegistro($nrorg, $nreventom, $nrtipomoviment, $nrtpeventoctbl, $nrestruturam, $nreventoctbl = null){
    
        $evento_contabil = $this->_evento_contabil->getFpaeventoctbl
        (
            $nrorg, 
            $nreventom, 
            $nrtipomoviment, 
            $nrtpeventoctbl, 
            $nrestruturam,
            $nreventoctbl
        );
        
        if(count($evento_contabil)>0)
            return true;
        else
            return false;
    }
    
    public function baixarPlanilhaTemplate ($row) {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
        $dtmescompetencia = $this->hcmParameters->getDtCompetencia();
        
        $events = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEventos($nrorg, $nrorgpadrao, $dtmescompetencia);
        
        foreach ($events as $evento) {
            $eventos[] = array("nrorg" => $evento->getNrorg(), "nreventom" => $evento->getNreventom(), "idtipoevento" => $evento->getIdtipoevento(), "nmeventoh" => $evento->historico->getNmeventoh());
        }
        
        $cabecalho = array(
            "Organização",
            "Nº do Evento",
            "Tipo",
            "Nome do Evento",
            "Conta de Débito",
            "Descrição de Débito",
            "Código Histórico Padrão Débito",
            "Descrição Histórico Padrão Débito",
            "Conta de Crédito",
            "Descrição de Crédito",
            "Código Histórico Padrão Cŕedito",
            "Descrição Histórico Padrão Cŕedito",
            "Tipo de Associação Contábil",
            "Código Despesa",
            "Nome Despesa"
        );
        return $this->exportacaoExcelService->gravarTemplate($eventos, $nrorg, $cabecalho, "Associacao Contabil");
        
    }
    
    public function consultaDespesa($row, $nrorg){
        $item = $row['CDDESPADMFAT'];
        if($item == null){
            return;
        }
        $despesa = $this->entityManager->getRepository(Repositories::DESPADMFATUR)->findOneBy(array('nrorg' => $nrorg, 'cddespadmfat' => $item));
        if(!isset($despesa)){
            $objEventCtbl = new Despadmfatur;
            $objEventCtbl->setNrorg($this->hcmParameters->getNrorgTrabalho());
            $objEventCtbl->setDtinclusao(DateUtil::getDataAtual());
            $objEventCtbl->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $objEventCtbl->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $objEventCtbl->setDtultatu(DateUtil::getDataAtual());
            $objEventCtbl->setCdoperultatu($this->hcmParameters->getCdoperador());
            $objEventCtbl->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $objEventCtbl->setcdDespadmFat($item);
            $objEventCtbl->setdsDespadmFat($row['DSDESPADMFAT']);
            $this->entityManager->persist($objEventCtbl);
            $this->entityManager->flush($objEventCtbl);
        }
    }
    
    public function importarPlanilha ($row) {
        try{
            $this->entityManager->beginTransaction();
            
            $contctbl = Repositories::CONTACONTABIL;
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
            $dtCompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
            
            $idImportacao = $row['IDOPCAOIMPORT'];
            
            $infos = $this->preparaImportacao($row);
            if($idImportacao === 'A') {
                foreach($infos as $key => $dados) {
                    //atualiza o valor da linha pra mostrar erros, se necessário
                    $linha = $key+2;
                    //filtra as associações informadas para salvar apenas as que possuem pelo menos uma conta contábil ou histórico associado com descrição
                    if( ( (!empty($dados['CONTDEBITO']) && !empty($dados['DESCDEBITO'])) || (empty($dados['CONTDEBITO']) && empty($dados['DESCDEBITO'])) ) && ( (!empty($dados['HISTPDRDEB']) && !empty($dados['DSCHPDRDEB'])) || (empty($dados['HISTPDRDEB']) && empty($dados['DSCHPDRDEB']))) && 
                        ( (!empty($dados['CONTCREDIT']) && !empty($dados['DESCCREDIT'])) || (empty($dados['CONTCREDIT']) && empty($dados['DESCCREDIT'])) ) && ( (!empty($dados['HISTPDRCRE']) && !empty($dados['DSCHPDRCRE'])) || (empty($dados['HISTPDRCRE']) && empty($dados['DSCHPDRCRE']))) &&
                          !(empty($dados['CONTDEBITO']) && empty($dados['HISTPDRDEB'])   && empty($dados['CONTCREDIT'])  && empty($dados['HISTPDRCRE']))){
                            $evento = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEvento($dados['EVENTO'], $nrorg, $nrorgpadrao, $dtCompetencia);
                            $existe = $this->verificaExistenciaCtbl($nrorg, $nrorgpadrao, $dados['EVENTO'], $dtCompetencia, $dados['CONTDEBITO'], $dados['HISTPDRDEB'], $dados['CONTCREDIT'], $dados['HISTPDRCRE'], $dados['CDDESPADMFAT'], $dados['NRTPEVENTOCTBL']);
                            if (empty($evento)) {
                                $this->mensagens[] = array("msg" => "Não foi encontrado um evento com o código ". $dados['EVENTO']);
                            }else if ($existe === true){
                                $this->mensagens[] = array("msg" => "Já existe uma Associação Contábil com os dados informados. Linha do erro: ". $linha);
                            } else {
                                // verificação de existência de histórico padrão de débito, caso não exista um com o código informado, é criado um novo
                                // só cria um novo histórico caso seja informado um código na planilha
                                $histpadrdeb = $this->entityManager->getRepository(Repositories::HISTORICOPADRAO)->findOneBy(array("nrorg" => array($nrorg, $nrorgpadrao), "cdhistpadr" => $dados['HISTPDRDEB']) );
                                if(empty($histpadrdeb) && !empty($dados['HISTPDRDEB'])) {
                                    $this->historicoPadraoService->setHistoricoPadrao($dados['HISTPDRDEB'], $dados['DSCHPDRDEB']);
                                }
                                
                                // verificacao da existencia de conta contabil de debito, caso nao exista uma nova conta é criada
                                // só cria uma nova conta caso seja informado um código na planilha
                                $contctbldeb = $this->entityManager->getRepository($contctbl)->findOneBy(array("nrorg" => array($nrorg, $nrorgpadrao), "cdcontctbl" => $dados['CONTDEBITO']) );
                                if(empty($contctbldeb) && !empty($dados['CONTDEBITO'])) {
                                    $this->contaContabilService->setContaCtbl($dados['CONTDEBITO'], $dados['DESCDEBITO']);
                                }
                                
                                // verificação de existência de histórico padrão de crédito, caso não exista um com o código informado, é criado um novo
                                // só cria um novo histórico caso seja informado um código na planilha
                                $histpadrcre = $this->entityManager->getRepository(Repositories::HISTORICOPADRAO)->findOneBy(array("nrorg" => array($nrorg, $nrorgpadrao), "cdhistpadr" => $dados['HISTPDRCRE']) );
                                if(empty($histpadrcre) && !empty($dados['HISTPDRCRE'])) {
                                    $this->historicoPadraoService->setHistoricoPadrao($dados['HISTPDRCRE'], $dados['DSCHPDRCRE']);
                                }
                                
                                // verificacao da existencia de conta contabil de credito, caso nao exista uma nova conta é criada
                                // só cria uma nova conta caso seja informado um código na planilha
                                $contctbldcre = $this->entityManager->getRepository($contctbl)->findOneBy(array("nrorg" => array($nrorg, $nrorgpadrao), "cdcontctbl" => $dados['CONTCREDIT']) );
                                if(empty($contctbldcre) && !empty($dados['CONTCREDIT'])) {
                                    $this->contaContabilService->setContaCtbl($dados['CONTCREDIT'], $dados['DESCCREDIT']);
                                }
                                //salva associação contabil
                                else if (strlen($dados['CDDESPADMFAT']) > 2) {
                                    $this->mensagens[] = array("msg" => "O limite de caracteres para o campo Código Despesa é de 2 dígitos". $linha);
                                }else if (strlen($dados['DSDESPADMFAT']) > 40) {
                                    $this->mensagens[] = array("msg" => "O limite de caracteres para o campo Nome Despesa é de 40 dígitos ". $linha);
                                }else {
                                    $objnrtpeventoctbl = $this->entityManager->getRepository(Repositories::FPA_TPEVENTOCTBL)->findBy(array("nrorg" => array($nrorg, $nrorgpadrao), "nrtpeventoctbl" => $dados['NRTPEVENTOCTBL']));
                                    if($objnrtpeventoctbl == null && $dados['NRTPEVENTOCTBL'] != null){
                                        $this->mensagens[] = array("msg" => "Não foi encontrado um tipo de Associação Contábil com o código informado, por favor, verifique e tente novamente.", "row" => $linha);
                                    } else {
                                        $this->consultaDespesa($dados, $nrorg);
                                        $contcredit = empty($newcontctblcre) ? $dados['CONTCREDIT'] : $newcontctblcre->getCdcontctbl();
                                        $contdebito = empty($newcontctbldeb) ? $dados['CONTDEBITO'] : $newcontctbldeb->getCdcontctbl();
                                        $histpdrcre = empty($newHistcre) ? $dados['HISTPDRCRE'] : $newHistcre->getCdhistpadr();
                                        $histpdrdeb = empty($newHistdeb) ? $dados['HISTPDRDEB'] : $newHistdeb->getCdhistpadr();
                                        $this->setEventoCtbl($dados['EVENTO'], $contcredit, $contdebito, $histpdrcre, $histpdrdeb, $dados['CDDESPADMFAT'], $dados['NRTPEVENTOCTBL']);
                                    }
                                }
                            }
                    } else if( !(empty($dados['CONTDEBITO']) && empty($dados['DESCDEBITO']) && empty($dados['HISTPDRDEB']) && empty($dados['DSCHPDRDEB']) && 
                                 empty($dados['CONTCREDIT']) && empty($dados['DESCCREDIT']) && empty($dados['HISTPDRCRE']) && empty($dados['DSCHPDRCRE'])) ) {
                        //retorna uma mensagem de erro informando onde foi informada uma descrição sem código na planilha
                        //só avisa se existirem códigos sem descrição
                                
                        if((!empty($dados['CONTDEBITO']) && empty($dados['DESCDEBITO']))){
                            $this->mensagens[] = array("msg" => "Não foi encontrado valor para a descrição da Conta Débito na linha ". $linha);                                    
                        }
                        
                        if((empty($dados['CONTDEBITO'])  && !empty($dados['DESCDEBITO']))){
                            $this->mensagens[] = array("msg" => "A descrição Conta Débito foi informada sem o código na linha ". $linha);                                    
                        } 
                        
                        if((!empty($dados['HISTPDRDEB']) && empty($dados['DSCHPDRDEB']))){
                            $this->mensagens[] = array("msg" => "Não foi encontrado valor para a descrição do Histórico Padrão Débito na linha ". $linha);                                    
                        }
                        
                        if((empty($dados['HISTPDRDEB'])  && !empty($dados['DSCHPDRDEB']))){
                            $this->mensagens[] = array("msg" => "A descrição do Histórico Padrão Débito foi informada sem código na linha ". $linha);                                    
                        } 
                        
                        if((!empty($dados['CONTCREDIT']) && empty($dados['DESCCREDIT']))){
                            $this->mensagens[] = array("msg" => "Não foi encontrado valor para a descrição do Conta Crédito na linha ". $linha); 
                        }
                        
                        if((empty($dados['CONTCREDIT'])  && !empty($dados['DESCCREDIT']))){
                            $this->mensagens[] = array("msg" => "A descrição da Conta Crédito foi informada sem código na linha ". $linha); 
                        } 
                        
                        if((!empty($dados['HISTPDRCRE']) && empty($dados['DSCHPDRCRE']))){
                            $this->mensagens[] = array("msg" => "Não foi encontrado valor para a descrição do Histórico Padrão Crédito na linha ". $linha); 
                        }
                        
                        if((empty($dados['HISTPDRCRE'])  && !empty($dados['DSCHPDRCRE']))){
                            $this->mensagens[] = array("msg" => "A descrição do Histórico Padrão Crédito foi informada sem código na linha ". $linha); 
                        }
                    }
                }
            } else if ($idImportacao === 'S') {
                foreach($infos as $key => $dados) {
                    $linha = $key+2;
                    if( ( (!empty($dados['CONTDEBITO']) && !empty($dados['DESCDEBITO'])) || (empty($dados['CONTDEBITO']) && empty($dados['DESCDEBITO'])) ) && ( (!empty($dados['HISTPDRDEB']) && !empty($dados['DSCHPDRDEB'])) || (empty($dados['HISTPDRDEB']) && empty($dados['DSCHPDRDEB']))) && 
                        ( (!empty($dados['CONTCREDIT']) && !empty($dados['DESCCREDIT'])) || (empty($dados['CONTCREDIT']) && empty($dados['DESCCREDIT'])) ) && ( (!empty($dados['HISTPDRCRE']) && !empty($dados['DSCHPDRCRE'])) || (empty($dados['HISTPDRCRE']) && empty($dados['DSCHPDRCRE']))) &&
                          !(empty($dados['CONTDEBITO']) && empty($dados['HISTPDRDEB'])   && empty($dados['CONTCREDIT'])  && empty($dados['HISTPDRCRE']))){
                            $obj = $this->entityManager->getRepository(Repositories::FPA_EVENTOCTBL)->findBy(array("nrorg" => array($nrorg, $nrorgpadrao), "nrestruturam" => null, "nreventom" => $dados['EVENTO'], "nrtpeventoctbl" => $dados['NRTPEVENTOCTBL']));
                            if(empty($obj)) {
                                $this->mensagens[] = array("msg" => "Não foi encontrado uma Associação Contábil ativa com o evento ". $dados['EVENTO'] ." Linha do erro: ". $linha);
                            }else if (count($obj) > 1) {
                                $this->mensagens[] = array("msg" => "Não foi possível substituir o evento ". $dados['EVENTO'] .", mais de um evento encontrado. Linha do erro: ". $linha);
                            }
                            else if (strlen($dados['CDDESPADMFAT']) > 2) {
                                $this->mensagens[] = array("msg" => "O limite de caracteres para o campo Código Despesa é de 2 dígitos". $linha);
                            }else if (strlen($dados['DSDESPADMFAT']) > 40) {
                                $this->mensagens[] = array("msg" => "O limite de caracteres para o campo Nome Despesa é de 40 dígitos ". $linha);
                            }else{
                                $objnrtpeventoctbl = $this->entityManager->getRepository(Repositories::FPA_TPEVENTOCTBL)->findBy(array("nrorg" => array($nrorg, $nrorgpadrao), "nrtpeventoctbl" => $dados['NRTPEVENTOCTBL']));
                                if($objnrtpeventoctbl == null && $dados['NRTPEVENTOCTBL'] != null){
                                    $this->mensagens[] = array("msg" => "Não foi encontrado um tipo de Associação Contábil com o código informado, por favor, verifique e tente novamente.", "row" => $linha);
                                } else {
                                    $this->consultaDespesa($dados, $nrorg);
                                    //filtra as associações informadas para salvar apenas as que possuem pelo menos uma conta contábil ou histórico associado com descrição
                                    $obj = $obj[0];
                                    $obj->setDtultatu(DateUtil::getDataAtual());
                                    $obj->setCdoperultatu($this->hcmParameters->getCdoperador());
                                    $obj->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                                    $obj->setCdcontctblcre($dados['CONTCREDIT']);
                                    $obj->setCdcontctbldeb($dados['CONTDEBITO']);
                                    $obj->setCdhistpadrcre($dados['HISTPDRCRE']);
                                    $obj->setCdhistpadrdeb($dados['HISTPDRDEB']);
                                    $obj->setNrtpeventoctbl($dados['NRTPEVENTOCTBL']);
                                    $obj->setcdDespadmFat($dados['CDDESPADMFAT']);
                                    $this->entityManager->persist($obj);
                                }
                            }
                    } else if( !(empty($dados['CONTDEBITO']) && empty($dados['DESCDEBITO']) && empty($dados['HISTPDRDEB']) && empty($dados['DSCHPDRDEB']) && 
                                     empty($dados['CONTCREDIT']) && empty($dados['DESCCREDIT']) && empty($dados['HISTPDRCRE']) && empty($dados['DSCHPDRCRE'])) ) {
                        //retorna uma mensagem de erro informando onde foi informada uma descrição sem código na planilha
                        //só avisa se existirem códigos sem descrição
                                
                        if((!empty($dados['CONTDEBITO']) && empty($dados['DESCDEBITO']))){
                            $this->mensagens[] = array("msg" => "Não foi encontrado valor para a descrição da Conta Débito na linha ". $linha);                                    
                        }
                        
                        if((empty($dados['CONTDEBITO'])  && !empty($dados['DESCDEBITO']))){
                            $this->mensagens[] = array("msg" => "A descrição Conta Débito foi informada sem o código na linha ". $linha);                                    
                        } 
                        
                        if((!empty($dados['HISTPDRDEB']) && empty($dados['DSCHPDRDEB']))){
                            $this->mensagens[] = array("msg" => "Não foi encontrado valor para a descrição do Histórico Padrão Débito na linha ". $linha);                                    
                        }
                        
                        if((empty($dados['HISTPDRDEB'])  && !empty($dados['DSCHPDRDEB']))){
                            $this->mensagens[] = array("msg" => "A descrição do Histórico Padrão Débito foi informada sem código na linha ". $linha);                                    
                        } 
                        
                        if((!empty($dados['CONTCREDIT']) && empty($dados['DESCCREDIT']))){
                            $this->mensagens[] = array("msg" => "Não foi encontrado valor para a descrição do Conta Crédito na linha ". $linha); 
                        }
                        
                        if((empty($dados['CONTCREDIT'])  && !empty($dados['DESCCREDIT']))){
                            $this->mensagens[] = array("msg" => "A descrição da Conta Crédito foi informada sem código na linha ". $linha); 
                        } 
                        
                        if((!empty($dados['HISTPDRCRE']) && empty($dados['DSCHPDRCRE']))){
                            $this->mensagens[] = array("msg" => "Não foi encontrado valor para a descrição do Histórico Padrão Crédito na linha ". $linha); 
                        }
                        
                        if((empty($dados['HISTPDRCRE'])  && !empty($dados['DSCHPDRCRE']))){
                            $this->mensagens[] = array("msg" => "A descrição do Histórico Padrão Crédito foi informada sem código na linha ". $linha); 
                        }
                    }
                }
            }
        
            //aplica as alterações
            $this->entityManager->flush();
            $this->entityManager->commit();
            return $this->mensagens;
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    private function verificarEventos($eventos, $nrorg, $nrorgpadrao, $dtmescompetencia) {
        $msg = array();
        if (!empty($eventos)) {
            foreach($eventos as $evento) {
                $event = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEvento($evento, $nrorgpadrao, $dtmescompetencia, $nrorg);
                if (empty($event)) {
                    $msg[] = array("msg" => "Não foi encontrado um evento com o código ".$evento);
                }
            }
        } else {
            throw new \Exception("Não há eventos");
        }
        
        if (!empty($msg)) {
            return $msg;
        } else {
            return false;
        }
    }
    
    public function preparaImportacao ($row) {
        $arquivo = $row['DSPATH'][0];
        
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $file = $this->httpKernel->factoryFromFileData($arquivo, FPA::getTempDirectory() . DIRECTORY_SEPARATOR);
        
        if (!$file->isFile()) {
            $this->mensagens[] = "Falha ao importar o arquivo.";
        }
        //Tratativa pra só incluir barra antes se for linux
        $caminhoNovo = (DIRECTORY_SEPARATOR == "/" ? DIRECTORY_SEPARATOR: "") . $file->getPathname();
        $this->validaExtensao($caminhoNovo, $arquivo['name']);
        return $this->preparaInformacao($caminhoNovo, $arquivo['name']);
    }
    
    private function validaExtensao($caminho, $arquivo) {
        $nome = preg_split("/(\\|\/|\.)/", $caminho);
        $extencao = strtoupper(end($nome));
        switch ($extencao) {
            case('XLS'):{
                return 1;
            }
            case('XLSX'):{
                return 2;
            }
            default:{
                $this->mensagens[] = "Extensão Inválida. (" . $arquivo . ")";
            }
        }
    }
    
    public function preparaInformacao($caminho, $nome) {
        try {
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $file = \PHPExcel_IOFactory::load($caminho);
            $file->setActiveSheetIndex(0);
            $eventos = $this->exportacaoExcelService->preparaInfoCtbl($file);
            $eventos = $this->validaFormatacaoEvento($eventos, $nome);
            
            return $eventos;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function validaFormatacaoEvento($eventos, $nomeArquivo){
        if (!empty($eventos)) {
            foreach ($eventos as $key => &$evento) {
                if (intval($evento['EVENTO']) == 0) {
                    $this->mensagens[] = array("msg" => "Evento inválido inserido na linha " . ++$key);
                } else {
                    $eventosValidos[] = $evento;
                }
            }
            $eventosValidos = $this->validaFormatacaoCtbl($eventosValidos, $nomeArquivo);
            return $eventosValidos;
        } else {
            $this->mensagens[] = array("msg" => "Não há eventos.");
            return false;
        }
    }
    
    private function validaFormatacaoCtbl($eventos, $nomeArquivo){
        foreach ($eventos as $key => $evento) {
            if (intval($evento['CONTDEBITO']) == 0 && $evento['CONTDEBITO'] !== null) {
                $this->mensagens[] = "Conta Contábil inválida inserido na linha " . ++$key . ", coluna E.";
            } else if (intval($evento['HISTPDRDEB']) == 0 && $evento['HISTPDRDEB'] !== null) {
                $this->mensagens[] = array("msg" => "Histórico Padrão inválido inserido na linha " . ++$key . ", coluna G.");
            } else if (intval($evento['CONTCREDIT']) == 0 && $evento['CONTCREDIT'] !== null) {
                $this->mensagens[] = array("msg" => "Conta Contábil inválida inserido na linha " . ++$key . ", coluna I.");
            } else if (intval($evento['HISTPDRCRE']) == 0 && $evento['HISTPDRCRE'] !== null) {
                $this->mensagens[] = array("msg" => "Histórico Padrão inválido inserido na linha " . ++$key . ", coluna K.");
            } else {
                $eventosValidos[] = $evento;
            }
        }
        return $eventosValidos;
    }
    
    public function verificaExistenciaCtbl ($nrorg, $nrorgpadrao, $nreventom, $dtcompetencia, $cdDebCtbl, $histDebCtbl, $cdCreCtbl, $histCreCtbl, $cdDespadmFat, $nrtpeventoctbl) {
        $result = $this->associacaoContabilRepository->getEventoCtbl($nrorg, $nrorgpadrao, $nreventom, $dtcompetencia, $cdDebCtbl, $histDebCtbl, $cdCreCtbl, $histCreCtbl, $nrtpeventoctbl);
        if (count($result) > 0) {
            return true;
        } else {
            return false;
        }
    }
    
    public function setEventoCtbl($nreventom, $contcredit, $contdebito, $histpdrcre, $histpdrdeb, $cdDespadmFat, $nrtpeventoctbl){
        $codigo = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_EVENTOCTBL', 1, 12, true);
        $objEventCtbl = new FpaEventoctbl;
        $objEventCtbl->setNrorg($this->hcmParameters->getNrorgTrabalho());
        $objEventCtbl->setDtinclusao(DateUtil::getDataAtual());
        $objEventCtbl->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $objEventCtbl->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $objEventCtbl->setDtultatu(DateUtil::getDataAtual());
        $objEventCtbl->setCdoperultatu($this->hcmParameters->getCdoperador());
        $objEventCtbl->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $objEventCtbl->setDtinivigencia(DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true));
        $objEventCtbl->setNreventom($nreventom);
        $objEventCtbl->setCdcontctblcre($contcredit);
        $objEventCtbl->setCdcontctbldeb($contdebito);
        $objEventCtbl->setCdhistpadrcre($histpdrcre);
        $objEventCtbl->setCdhistpadrdeb($histpdrdeb);
        $objEventCtbl->setNrtipomoviment(null);
        $objEventCtbl->setNrtpeventoctbl($nrtpeventoctbl);
        $objEventCtbl->setNreventoctbl($codigo);
        $objEventCtbl->setcdDespadmFat($cdDespadmFat);
        $objEventCtbl->setIdvincativo('S');
        $this->entityManager->persist($objEventCtbl);
        $this->entityManager->flush($objEventCtbl);
    }
    
} 

