<?php
namespace HCM\PTO\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\{ Repositories, Util, HCMParameters, DateUtil, Messages, DoctrineEntityUtil };
use HCM\PTO\Services\{ OcorrenciaFrequenciaService, ApuracaoFrequenciaService };
use HCM\Geral\Repository\VinculoRepository;

class RoteiroParamFrequenciaService {
    
    /* Ocorrências */
    const DESC_BANCOHORASPROG_PREFIX = 'Ocorrência Parametrização';
    
    /* Situação de Frequência */
    const DSSITUFREQUENCIA_QUITA_POSITIVO   = 'Quitar Saldo Positivo Parametrização';
    const DSABREVIACAO_QUITA_POSITIVO       = 'PARAM_CRED_ACUMULADO';
    const DSSITUFREQUENCIA_QUITA_NEGATIVO   = 'Quitar Saldo Negativo Parametrização';
    const DSABREVIACAO_QUITA_NEGATIVO       = 'PARAM_DEB_ACUMULADO';
    
    /* Params Roteiro Frequência */
    const RPF_ALL_FIELDS = [
        'NRTIPOVINCULOM', 'NMTIPOVINCULOH', 'NRVINCULOM_LIST', 'NMVINCULOM_LIST', 'NRESTRUTLEGAL_LIST', 'NMESTRUTLEGAL_LIST', 'NRESTRUTGEREN_LIST', 'NMESTRUTGEREN_LIST', 'NRESTRUTSIND_LIST', 'NMESTRUTSIND_LIST',
        'DSBANCOHORAS', 'DTINICIOBANCOHORAS', 'DTFIMBANCOHORAS', 'CRIA_PARAM_BANCOHORA', 'roteiroParamFrequencia_step3_cadPeriodo', 'ACUMULA_SALDO', 'GERA_VRCOEFICIENTEACRESCIMO',
        'VALOR_VRCOEFICIENTEACRESCIMO', 'GERA_AUSENC_COMP', 'VALOR_AUSENC_COMP', 'GERA_QUITA_SALDO_OU_DEB', 'VALOR_VRCOEFICIENTEACRESCIMO_QUITA_SALDO_OU_DEB', 'NMEVENTOH_QUITA_CRED_ACUMULADO',
        'NREVENTOM_QUITA_CRED_ACUMULADO', 'NMEVENTOH_QUITA_DEBITO', 'NREVENTOM_QUITA_DEBITO', 'GERA_COMPLEMENTA_TRABALHO', 'PROJETA_INTERVALO', 'UTILIZA_TOLERANCIA', 'VALOR_TOLERANCIA',
        'GERA_HE_LIMITESEPHEXTRA', 'VALOR_LIMITESEPHEXTRA', 'GERA_HE_MAXHRTRAB', 'VALOR_MAXHRTRAB', 'NRSITFREQMAXHRTRAB', 'DSSITFREQMAXHRTRAB', 'roteiroParamFrequencia_step6_cadEventosHE',
        'HRINIPERIODONOTURNO', 'HRFIMPERIODONOTURNO', 'NREVENTOM_ADICIONAL_NOTURNO', 'NMEVENTOH_ADICIONAL_NOTURNO',
    ];
    
    const RPF_ARRAY_FIELDS = [
        'NRVINCULOM_LIST', 'NMVINCULOM_LIST', 'NRESTRUTLEGAL_LIST', 'NMESTRUTLEGAL_LIST', 'NRESTRUTGEREN_LIST', 'NMESTRUTGEREN_LIST', 'NRESTRUTSIND_LIST', 'NMESTRUTSIND_LIST',
        'roteiroParamFrequencia_step3_cadPeriodo', 'roteiroParamFrequencia_step6_cadEventosHE',
    ];
    
    /* Defaults */
    const RPF_ASSOC_SF_DEFAULTS = [
        'F' => [
            'NRCALCSITUFREQ' => [ '1AS' => [ 3700, 4400 ], 'SEG' => [ 3800, 4500 ] ],
            '3700' => [ 'DSSITUFREQUENCIA'  => 'Hora Extra Diurna Folga I Parametrização',          'DSABREVIACAO' => 'PARAM_EX_DIA_FOLGA_1AS' ],
            '4400' => [ 'DSSITUFREQUENCIA'  => 'Hora Extra Noturna Folga I Parametrização',         'DSABREVIACAO' => 'PARAM_EX_NOT_FOLGA_1AS' ],
            '3800' => [ 'DSSITUFREQUENCIA'  => 'Hora Extra Diurna Folga II Parametrização',         'DSABREVIACAO' => 'PARAM_EX_NOT_FOLGA_1AS' ],
            '4500' => [ 'DSSITUFREQUENCIA'  => 'Hora Extra Noturna Folga II Parametrização',        'DSABREVIACAO' => 'PARAM_EX_NOT_FOLGA_1AS' ],
        ],
        'C' => [
            'NRCALCSITUFREQ' => [ '1AS' => [ 5200, 5900 ], 'SEG' => [ 5300, 6000 ] ],
            '5200' => [ 'DSSITUFREQUENCIA'  => 'Hora Extra Diurna Compensado I Parametrização',     'DSABREVIACAO'  => 'PARAM_EX_DIA_SABADO_1AS' ],
            '5900' => [ 'DSSITUFREQUENCIA'  => 'Hora Extra Noturna Compensado I Parametrização',    'DSABREVIACAO'  => 'PARAM_EX_NOT_SABADO_1AS' ],
            '5300' => [ 'DSSITUFREQUENCIA'  => 'Hora Extra Diurna Compensado II Parametrização',    'DSABREVIACAO'  => 'PARAM_EX_NOT_SABADO_1AS' ],
            '6000' => [ 'DSSITUFREQUENCIA'  => 'Hora Extra Noturna Compensado II Parametrização',   'DSABREVIACAO'  => 'PARAM_EX_NOT_SABADO_1AS' ],
        ],
        'N' => [
            'NRCALCSITUFREQ' => [ '1AS' => [ 2200, 2900 ], 'SEG' => [ 2300, 3000 ] ],
            '2200' => [ 'DSSITUFREQUENCIA'  => 'Hora Extra Diurna Normal I Parametrização',         'DSABREVIACAO'  => 'PARAM_EX_DIA_NORMAL_1AS' ],
            '2900' => [ 'DSSITUFREQUENCIA'  => 'Hora Extra Noturna Normal I Parametrização',        'DSABREVIACAO'  => 'PARAM_EX_NOT_NORMAL_1AS' ],
            '2300' => [ 'DSSITUFREQUENCIA'  => 'Hora Extra Diurna Normal II Parametrização',        'DSABREVIACAO'  => 'PARAM_EX_NOT_NORMAL_1AS' ],
            '3000' => [ 'DSSITUFREQUENCIA'  => 'Hora Extra Noturna Normal II Parametrização',       'DSABREVIACAO'  => 'PARAM_EX_NOT_NORMAL_1AS' ],
        ],
        'FERIADO' => [
            'NRCALCSITUFREQ' => [ '1AS' => [ 6700, 7400 ], 'SEG' => [ 6800, 7500 ] ],
            '6700' => [ 'DSSITUFREQUENCIA'  => 'Hora Extra Diurna Feriado I Parametrização',        'DSABREVIACAO'  => 'PARAM_EX_DIA_FERIADO_1AS' ],
            '7400' => [ 'DSSITUFREQUENCIA'  => 'Hora Extra Noturna Feriado I Parametrização',       'DSABREVIACAO'  => 'PARAM_EX_NOT_FERIADO_1AS' ],
            '6800' => [ 'DSSITUFREQUENCIA'  => 'Hora Extra Diurna Feriado II Parametrização',       'DSABREVIACAO'  => 'PARAM_EX_NOT_FERIADO_1AS' ],
            '7500' => [ 'DSSITUFREQUENCIA'  => 'Hora Extra Noturna Feriado II Parametrização',      'DSABREVIACAO'  => 'PARAM_EX_NOT_FERIADO_1AS' ],
        ],
    ];
    
    const RPF_ASSOC_SF_ADICIONAL_NOTURNO = [
        'NRCALCSITUFREQ'    => 9000,
        'DSSITUFREQUENCIA'  => 'Adicional Noturno Parametrização',
        'DSABREVIACAO'      => 'PARAM_ADICIONAL_NOTURNO',
    ];

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, DoctrineEntityUtil $doctrineEntityUtil, VinculoRepository $vinculoRepository) {
        $this->entityManager            = $entityManager;
        $this->hcmParameters            = $hcmParameters;
        $this->doctrineEntityUtil       = $doctrineEntityUtil;
        $this->vinculoRepository        = $vinculoRepository;
        $this->gpeEscalatrabhRepository = Repositories::GPE_ESCALATRABH;
    }
    
    /**
     *  Rotina que realiza o cadastro das informações da parametrização de frequência. A rotina foi preparada de modo que qualquer excecao ao longo de sua execucao
     * impeca a sincronizacao das informacoes com a base de dados
     * 
     *  @param  Array   $row
     * 
     *  @return bool
     * 
     */
    public function setRoteiroParamFrequencia($row) {
        $this->setRoteiroParamCheckInput($row);
        
        $row = $this->setEmptyRoteiroParamFieldsArray($row);
        
        $this->entityManager->getConnection()->beginTransaction();
        
        try {
            
            // Tabela PTO_SITUFREQUENCIA nao possui NRORG na chave e precisa ser atualizada antes da utilizacao da entidade NOVOCODIGO
            Util::updateNovoCodigo('PTO_SITUFREQUENCIA', false);
            
            /* [ETAPA 1] Busca vínculos dentro do escopo informado */
            $vinculos = $this->vinculoRepository->getVinculosByEstruturas(
                $this->hcmParameters->getNrorgTrabalho(),
                $this->hcmParameters->getDtCompetencia(),
                $row['NRESTRUTLEGAL_LIST'],
                $row['NRESTRUTGEREN_LIST'],
                $row['NRVINCULOM_LIST'],
                $row['NRTIPOVINCULOM'],
                $row['NRESTRUTSIND_LIST']
            );
            
            if (empty($vinculos)) {
                throw new \Exception(Messages::RPF_VINCULOS_NOT_FOUND);
            }
            
            // Escalas de trabalho (GPE_ESCALATRABH) associadas aos vinculos do escopo
            $escalasTrabh = $this->getEscalatrabhVinculos($vinculos);
            
            /* [ETAPA 2] Dados Iniciais - Cria Banco de Horas*/
            $bancoHoras = $this->setBancoHoras($row['DSBANCOHORAS'], $row['DTINICIOBANCOHORAS'], $row['DTFIMBANCOHORAS'], false);
            
            /* [ETAPA 3] Banco de Horas - Cria Periodos de Banco de Horas */
            $periodosBancoHoras = $this->setBancoHorasPeriodoEtapa3RPF(
                $bancoHoras->getNrbancohoras(),
                $row['roteiroParamFrequencia_step3_cadPeriodo'],
                $row['DTINICIOBANCOHORAS'],
                $row['DTFIMBANCOHORAS'],
                $row['CRIA_PARAM_BANCOHORA'],
                $row['ACUMULA_SALDO']
            );
            
            /* [ETAPA 4] Detalhamento da Apuração - Cria Programações de Ocorrência para o Banco de Horas */
            $this->setBancoHorasProgEtapa4RPF(
                $bancoHoras->getNrbancohoras(),
                $row['DTINICIOBANCOHORAS'],
                $row['ACUMULA_SALDO'],
                $row['GERA_VRCOEFICIENTEACRESCIMO'],
                $row['VALOR_VRCOEFICIENTEACRESCIMO'],
                $row['GERA_AUSENC_COMP'],
                $row['VALOR_AUSENC_COMP'],
                $row['GERA_QUITA_SALDO_OU_DEB'],
                $row['VALOR_VRCOEFICIENTEACRESCIMO_QUITA_SALDO_OU_DEB'],
                $row['NREVENTOM_QUITA_CRED_ACUMULADO'],
                $row['NREVENTOM_QUITA_DEBITO'],
                $periodosBancoHoras,
                $row['DSBANCOHORAS']
            );
            
            /* [ETAPA 5] Jornada de trabalho */
            $this->setBancoHorasProgEtapa5RPF(
                $bancoHoras->getNrbancohoras(),
                $row['DTINICIOBANCOHORAS'],
                $row['GERA_COMPLEMENTA_TRABALHO'],
                $row['DSBANCOHORAS']
            );
            
            $this->alterEscalatrabhEtapa5RPF(
                $escalasTrabh,
                $row['PROJETA_INTERVALO']
            );
            
            $this->alterParamOrgEtapa5RPF(
                $row['UTILIZA_TOLERANCIA'],
                $row['VALOR_TOLERANCIA']
            );
            
            /* [ETAPA 6] Horas Extras */
            $this->alterEscalatrabhEtapa6RPF(
                $escalasTrabh,
                $row['GERA_HE_LIMITESEPHEXTRA'],
                $row['VALOR_LIMITESEPHEXTRA'],
                $row['GERA_HE_MAXHRTRAB'],
                $row['VALOR_MAXHRTRAB'],
                $row['NRSITFREQMAXHRTRAB']
            );
            
            $this->setSituacoesFrequenciaEventosEtapa6RPF(
                $row['roteiroParamFrequencia_step6_cadEventosHE'],
                $row['DTINICIOBANCOHORAS'],
                $row['GERA_HE_LIMITESEPHEXTRA'],
                $row['VALOR_LIMITESEPHEXTRA']
            );
            
            /* [ETAPA 7] Adicional Noturno */
            $this->alterEscalatrabhEtapa7RPF(
                $escalasTrabh,
                $row['HRINIPERIODONOTURNO'],
                $row['HRFIMPERIODONOTURNO']
            );
            
            $this->setSituacoesFrequenciaEventosEtapa7RPF(
                $row['NREVENTOM_ADICIONAL_NOTURNO'],
                $row['DTINICIOBANCOHORAS']
            );
            
            /* [FINAL] Cria Associações de Banco de Horas para os colaboradores do escopo */
            $this->criaAlterBancoHorasEtapaFinalRPF(
                $bancoHoras->getNrbancohoras(),
                $vinculos,
                $row['DTINICIOBANCOHORAS']
            );
            
            // Persiste entidades 'mestre' pendentes
            $this->doctrineEntityUtil->persistPending();
            
            $this->entityManager->flush();
            $this->entityManager->getConnection()->commit();
            
            return true;
            
        } catch (\Exception $e) {
            $this->entityManager->getConnection()->rollBack();
            throw $e;
        }
        
    }
    
    /**
     *  Rotina que preenche campos esperados pelas rotinas da parametrizacao no input
     * 
     *  @param  Array   $row
     * 
     *  @throws \Exception
     * 
     */
    public function setEmptyRoteiroParamFieldsArray(
        Array   $row
    ) {
        $rowFinal = Util::setEmptyFieldsArray($row, self::RPF_ALL_FIELDS);
        $rowFinal = Util::setEmptyFieldsArray($rowFinal, self::RPF_ARRAY_FIELDS, [], true);
        
        return $rowFinal;
    }
    
    /**
     *  Rotina que valida se as informações que vão ser inseridas na parametrização são válidas
     * 
     *  @param  Array   $row
     * 
     *  @throws \Exception
     * 
     */
    public function setRoteiroParamCheckInput(Array $row) {
        $requiredFields = ['DSBANCOHORAS', 'DTINICIOBANCOHORAS', 'DTFIMBANCOHORAS', 'ACUMULA_SALDO'];
        $atLeastOneFields = ['NRTIPOVINCULOM', 'NRVINCULOM_LIST', 'NRESTRUTLEGAL_LIST', 'NRESTRUTGEREN_LIST', 'NRESTRUTSIND_LIST'];
        Util::checkRequiredFields($row, $requiredFields);
        Util::checkAtLeastOneFields($row, $atLeastOneFields);
    }
    
    /**
     *  Rotina que cria os periodos de banco de horas, de acordo com as informacoes da Etapa 3. O array de periodos eh retornado
     * 
     *  @param $nrBancohoras
     *  @param $periodosBancoHoras
     *  @param $dtIniciobancohoras
     *  @param $dtFimbancohoras
     *  @param $criaParamBancohora
     *  @param $acumulaSaldo
     * 
     *  @return Array
     * 
     */
    public function setBancoHorasPeriodoEtapa3RPF(
            $nrBancohoras,
            $periodosBancoHoras,
            $dtIniciobancohoras,
            $dtFimbancohoras,
            $criaParamBancohora,
            $acumulaSaldo
    ) {
        
        $periodosBancoHoras = is_array($periodosBancoHoras) ? $periodosBancoHoras : [];
            
        // Os perídos são criados apenas se o parâmetro dos fechamentos períodos estiver marcado
        if ($criaParamBancohora == 'S') {
            foreach ($periodosBancoHoras as $periodo) {
                $this->setBancoHorasPeriodo(
                    $nrBancohoras,
                    $periodo['DTINICIOPERIODOBHORAS'],
                    $periodo['DTFIMPERIODOBHORAS']
                );
            }
        } else {
            // Cria período padrão se não houverem fechamentos períodicos mas houver saldo de banco
            if ($acumulaSaldo == 'S') {
                $periodoPadraoBancoSaldo = [
                    'DTINICIOPERIODOBHORAS' => $dtIniciobancohoras,
                    'DTFIMPERIODOBHORAS'    => $dtFimbancohoras,
                ];
                
                $this->setBancoHorasPeriodo(
                    $nrBancohoras,
                    $periodoPadraoBancoSaldo['DTINICIOPERIODOBHORAS'],
                    $periodoPadraoBancoSaldo['DTFIMPERIODOBHORAS']
                );
            }
            
            // Impede que os períodos sejam considerados para demais processos
            $periodosBancoHoras = [];
        }
        
        return $periodosBancoHoras;
    }
    
    /**
     *  Rotina que realiza o cadastro de informações de ocorrências de banco de horas de acordo com as informações cadastradas na parmetrização
     * 
     *  @param  $nrBancoHoras
     *  @param  $dtInicio
     *  @param  $acumulaSaldo
     *  @param  $geraVrcoeficienteacrescimo
     *  @param  $valorVrcoeficienteacrescimo
     *  @param  $geraAusencComp
     *  @param  $valorAusencComp
     *  @param  $geraQuitaSaldoOuDeb
     *  @param  $valorVrcoeficienteacrescimoQuitaSaldoOuDeb
     *  @param  $nreventohQuitaCredAcumulado
     *  @param  $nreventohQuitaDebito
     *  @param  $periodosBancoHoras                             = []
     *  @param  $dsBancoHoras                                   = null
     *  @param  $dsBancoHorasProg                               = null
     * 
     */
    public function setBancoHorasProgEtapa4RPF(
        $nrBancoHoras,
        $dtInicio,
        $acumulaSaldo,
        $geraVrcoeficienteacrescimo,
        $valorVrcoeficienteacrescimo,
        $geraAusencComp,
        $valorAusencComp,
        $geraQuitaSaldoOuDeb,
        $valorVrcoeficienteacrescimoQuitaSaldoOuDeb,
        $nrEventomQuitaCredAcumulado,
        $nrEventomQuitaDebito,
        $periodosBancoHoras                                 = [],
        $dsBancoHorasProg                                   = null
    ) {
        $descricaoOcorrencias = $this->getDescricaoOcorrencia($dsBancoHorasProg);
        
        /* Ocorrências de autorizar horas a folgar */
        if ($acumulaSaldo == 'S') {
            // Cria ocorrência de autorizar horas a folgar e autorizar horas a folgar com acréscimo
            $this->setBancoHorasProg(
                $nrBancoHoras,
                OcorrenciaFrequenciaService::TIPOPROG_HORAS_FOLGAR,
                $dtInicio,
                null,
                null,
                $descricaoOcorrencias,
                null
            );
            
            $coeficienteAcrHorasFolgar = $geraVrcoeficienteacrescimo == 'S' ? $valorVrcoeficienteacrescimo : null;
            
            $this->setBancoHorasProg(
                $nrBancoHoras,
                OcorrenciaFrequenciaService::TIPOPROG_HORAS_FOLGAR_ACRESC,
                $dtInicio,
                null,
                $coeficienteAcrHorasFolgar,
                $descricaoOcorrencias,
                null
            );
        }
        
        /* Ocorrências de compensar atrasos */
        if ($geraAusencComp == 'S' && !empty($valorAusencComp)) {
            $ausencCompAvailable = [
                'AUSENC_COMP_SE_CRED'           => OcorrenciaFrequenciaService::TIPOPROG_AUSENC_COMP_SE_CRED,
                'AUSENCIA_COMPENSAR'            => OcorrenciaFrequenciaService::TIPOPROG_AUSENCIA_COMPENSAR,
                'AUSENC_COMP_ATRASOS_SOMENTE'   => OcorrenciaFrequenciaService::TIPOPROG_AUSENC_COMP_ATRASOS_SOMENTE,
            ];
            
            // Cria ocorrência de compensar de acordo com $valorAusencComp
            if (array_key_exists($valorAusencComp, $ausencCompAvailable)) {
                $this->setBancoHorasProg(
                    $nrBancoHoras,
                    $ausencCompAvailable[$valorAusencComp],
                    $dtInicio,
                    null,
                    null,
                    $descricaoOcorrencias,
                    null
                );
            }
        }
        
        /* Ocorrências de quitar horas em saldo */
        if ($geraQuitaSaldoOuDeb == 'S' && count($periodosBancoHoras) > 0) {
            $nrSituFreqCred = OcorrenciaFrequenciaService::TIPOPROG_QUITA_CRED_ACUMULADO;
            $nrSituFreqDeb  = OcorrenciaFrequenciaService::TIPOPROG_QUITA_DEBITO;
            
            // Cria situações de frequência com parâmetros dos eventos informados caso forem informados
            if (!empty($nrEventomQuitaCredAcumulado)) {
                $situFreqCred = $this->setSituFrequencia(
                    self::DSSITUFREQUENCIA_QUITA_POSITIVO,
                    self::DSABREVIACAO_QUITA_POSITIVO,
                    $dtInicio,
                    null
                );
                
                $this->setSituFreqParam(
                    $situFreqCred->getNrsitufrequencia(),
                    $nrEventomQuitaCredAcumulado
                );
                
                $nrSituFreqCred = $situFreqCred->getNrsitufrequencia();
            }
            
            if (!empty($nrEventomQuitaDebito)) {
                $situFreqDeb = $this->setSituFrequencia(
                    self::DSSITUFREQUENCIA_QUITA_NEGATIVO,
                    self::DSABREVIACAO_QUITA_NEGATIVO,
                    $dtInicio,
                    null
                );
                
                $this->setSituFreqParam(
                    $situFreqDeb->getNrsitufrequencia(),
                    $nrEventomQuitaDebito
                );
                
                $nrSituFreqDeb = $situFreqDeb->getNrsitufrequencia();
            }
            
            // Cria ocorrência pra quitar com a data fim de cada período
            foreach ($periodosBancoHoras as $periodo) {
                $this->setBancoHorasProg(
                    $nrBancoHoras,
                    OcorrenciaFrequenciaService::TIPOPROG_QUITA_CRED_ACUMULADO,
                    $periodo['DTFIMPERIODOBHORAS'],
                    $periodo['DTFIMPERIODOBHORAS'],
                    $valorVrcoeficienteacrescimoQuitaSaldoOuDeb,
                    $descricaoOcorrencias,
                    $nrSituFreqCred
                );
                
                $this->setBancoHorasProg(
                    $nrBancoHoras,
                    OcorrenciaFrequenciaService::TIPOPROG_QUITA_DEBITO,
                    $periodo['DTFIMPERIODOBHORAS'],
                    $periodo['DTFIMPERIODOBHORAS'],
                    $valorVrcoeficienteacrescimoQuitaSaldoOuDeb,
                    $descricaoOcorrencias,
                    $nrSituFreqDeb
                );
            }
        }
    }
    
    /**
     *  Rotina que realiza o cadastro de ocorrencias de banco de horas informadas na Etapa 5
     * 
     *  @param  $nrBancoHoras
     *  @param  $dtInicio
     *  @param  $geraComplementaTrabalho
     *  @param  $dsBancoHorasProg           = null
     * 
     */
    public function setBancoHorasProgEtapa5RPF(
        $nrBancoHoras,
        $dtInicio,
        $geraComplementaTrabalho,
        $dsBancoHorasProg           = null
    ) {
        $descricaoOcorrencia = $this->getDescricaoOcorrencia($dsBancoHorasProg);
        
        if ($geraComplementaTrabalho == 'S') {
            $this->setBancoHorasProg(
                $nrBancoHoras,
                OcorrenciaFrequenciaService::TIPOPROG_COMPLEMENTA_TRABALHO,
                $dtInicio,
                null,
                null,
                $descricaoOcorrencia,
                null
            );
        }
    }
    
    /**
     *  Rotina que busca o historico mais recente das escalas associadas os vinculos informados
     * 
     *  @param  Array   $vinculos
     * 
     *  @return Array   [GpeEscalatrabh]
     */
    public function getEscalatrabhVinculos(
        Array   $vinculos
    ) {
        $competencia = $this->hcmParameters->getDtCompetencia();
        $nrOrg = $this->hcmParameters->getNrorgTrabalho();
        
        $escalasTrabh = [];
        
        foreach ($vinculos as $vinculo) {
            $alteEscala = $this->entityManager->getRepository(Repositories::GPE_ALTEESCALA)->getAlteEscalaByCompetencia(
                $vinculo['NRVINCULOM'], $nrOrg, $competencia
            );
            
            if (empty($alteEscala) || empty($alteEscala['NRESCALATRABM'])) {
                continue;
            }
            
            $escalaTrabh = $this->entityManager->getRepository(Repositories::GPE_ESCALATRABH)->retornaUltimoHistorico(
                $alteEscala['NRESCALATRABM'], $nrOrg
            );
            
            if (empty($escalaTrabh)) {
                continue;
            }
            
            $escalasTrabh[] = $escalaTrabh;
        }
        
        return $escalasTrabh;
    }
    
    /**
     *  Rotina que realiza alteracoes nas escalas de trabalho informadas de acordo com informacoes da Etapa 5
     * 
     *  @param  $escalasTrabh
     *  @param  $projetaIntervalo
     * 
     */
    public function alterEscalatrabhEtapa5RPF(
        Array   $escalasTrabh,
        string  $projetaIntervalo
    ) {
        $competencia = $this->hcmParameters->getDtCompetencia();
        $nrOrg = $this->hcmParameters->getNrorgTrabalho();
        
        $idProjetaIntervaloAvailable = ['S', 'N'];
        if (!in_array($projetaIntervalo, $idProjetaIntervaloAvailable)) {
            return false;
        }
        
        foreach ($escalasTrabh as $escalaTrabh) {
            
            if (!$escalaTrabh instanceof $this->gpeEscalatrabhRepository) {
                continue;
            }
            
            $escalaTrabh->setIdprojetaintervalo($projetaIntervalo);
            
            $this->entityManager->persist($escalaTrabh);
        }
        
        return true;
    }
    
    /**
     *  Rotina que altera informacoes nos parametros da organizacao (GPE_PARAMORG) de acordo com informacoes da Etapa 5
     * 
     *  @param  $utilizaTolerancia
     *  @param  $valorTolerancia    = null
     * 
     */
    public function alterParamOrgEtapa5RPF(
        string  $utilizaTolerancia,
                $valorTolerancia    = null
    ) {
        $defaultTolerancia = '00:00';
        
        $paramOrg = $this->entityManager->getRepository(Repositories::GPE_PARAMORG)->findOneBy(
            array('nrorg' => $this->hcmParameters->getNrorgTrabalho())
        );
        
        if (empty($paramOrg)) {
            return false;
        }
        
        $valorTolerancia = $utilizaTolerancia == 'S' && !empty($valorTolerancia) ? $valorTolerancia : $defaultTolerancia;
        
        $paramOrg->setHrtempointervaloignorado($valorTolerancia);
        $this->entityManager->persist($paramOrg);
        
        return true;
    }
    
    /**
     *  Rotina que realiza alteracoes no historico mais recente das escalas informadas, de acordo com informacoes da Etapa 6
     * 
     *  @param  $escalasTrabh,
     *  @param  $geraHeLimitesephextra,
     *  @param  $valorLimitesephextra,
     *  @param  $geraHeMaxhrtrab,
     *  @param  $valorMaxhrtrab,
     *  @param  $nrSitfreqmaxhrtrab
     * 
     */
    public function alterEscalatrabhEtapa6RPF(
        Array   $escalasTrabh,
                $geraHeLimitesephextra,
                $valorLimitesephextra,
                $geraHeMaxhrtrab,
                $valorMaxhrtrab,
                $nrSitfreqmaxhrtrab
    ) {
        if (count($escalasTrabh) == 0) {
            return false;
        }
        
        $defaultLimitesephextraOnNotS = '2359';
        $defaultLimitesephextra = '0000';
        
        $valorLimitesephextraFinal  = !empty($valorLimitesephextra) ? Util::timeStringClearColon($valorLimitesephextra) : null;
        $valorMaxhrtrabFinal        = !empty($valorMaxhrtrab) ? Util::timeStringClearColon($valorMaxhrtrab) : null;
        $nrSitfreqmaxhrtrabFinal    = $nrSitfreqmaxhrtrab;
        
        if ($geraHeLimitesephextra != 'S') {
            $valorLimitesephextraFinal = $defaultLimitesephextraOnNotS;
        }
        
        $valorLimitesephextraFinal = empty($valorLimitesephextraFinal) ? $defaultLimitesephextra : $valorLimitesephextraFinal;
        
        if ($geraHeMaxhrtrab != 'S') {
            $valorMaxhrtrabFinal = null;
            $nrSitfreqmaxhrtrabFinal = null;
        }
        
        foreach ($escalasTrabh as $escalaTrabh) {
            if (!$escalaTrabh instanceof $this->gpeEscalatrabhRepository) {
                continue;
            }
            
            $escalaTrabh->setQthrlimitesephextra($valorLimitesephextraFinal);
            $escalaTrabh->setQthrmaxhrtrab($valorMaxhrtrabFinal);
            $escalaTrabh->setNrsitfreqmaxhrtrab($nrSitfreqmaxhrtrabFinal);
            
            $this->entityManager->persist($escalaTrabh);
        }
        
        return true;
    }
    
    /**
     *  Rotina que realiza a criacao e associacao de situacoes a eventos e a situacoes de frequencia padrao, a partir de informacoes da Etapa 6. Sao criadas novas situacoes, associadas
     * a situacoes padrao e associadas aos eventos de folha informados. Sao esperados eventos conforme definicao em self::RPF_ASSOC_SF_DEFAULTS
     * 
     *  @param  $eventosFrequencia
     *  @param  $dtIniVigencia
     * 
     */
    public function setSituacoesFrequenciaEventosEtapa6RPF(
        Array   $eventosFrequencia,
                $dtIniVigencia,
                $geraHeLimitesephextra,
                $valorLimitesephextra
    ) {
        
        // Determina o tipo de HE a ser gerada, de acordo com parametros informados
        $tipoHE = $geraHeLimitesephextra == 'S' && !empty($valorLimitesephextra) ? ApuracaoFrequenciaService::TIPO_SEG : ApuracaoFrequenciaService::TIPO_1AS;
        
        foreach ($eventosFrequencia as $eventoFrequencia) {
            if (!isset($eventoFrequencia['NREVENTOM_IDTIPOHORARIO']) || !isset($eventoFrequencia['IDTIPOHORARIO'])) {
                continue;
            }
            
            $defaultValues = self::RPF_ASSOC_SF_DEFAULTS[$eventoFrequencia['IDTIPOHORARIO']];
            if (empty($defaultValues)) {
                continue;
            }
            
            $nrCalculosHE = $defaultValues['NRCALCSITUFREQ'][$tipoHE];
            
            foreach ($nrCalculosHE as $nrCalculoSituFreq) {
                
                $calculoDefaults = $defaultValues[$nrCalculoSituFreq];
                
                $this->setAssocEventoSituFreq(
                    $calculoDefaults['DSSITUFREQUENCIA'],
                    $calculoDefaults['DSABREVIACAO'],
                    $dtIniVigencia,
                    $eventoFrequencia['NREVENTOM_IDTIPOHORARIO'],
                    $nrCalculoSituFreq
                );
            }
        }
    }
    
    /**
     *  Rotina que altera as escalas de trabalho informadas de acordo com informacoes da Etapa 7
     * 
     *  @param  $escalasTrabh,
     *  @param  $hrIniperiodonoturno,
     *  @param  $hrFimperiodonoturno
     * 
     */
    public function alterEscalatrabhEtapa7RPF(
        Array   $escalasTrabh,
                $hrIniperiodonoturno,
                $hrFimperiodonoturno
    ) {
        
        $hrIniperiodonoturnoFinal = Util::timeStringClearColon($hrIniperiodonoturno);
        $hrFimperiodonoturnoFinal = Util::timeStringClearColon($hrFimperiodonoturno);
        
        foreach ($escalasTrabh as $escalaTrabh) {
            
            if (!$escalaTrabh instanceof $this->gpeEscalatrabhRepository) {
                continue;
            }
            
            $escalaTrabh->setHriniperiodonoturno($hrIniperiodonoturnoFinal);
            $escalaTrabh->setHrfimperiodonoturno($hrFimperiodonoturnoFinal);
            
            $this->entityManager->persist($escalaTrabh);
        }
    }
    
    /**
     *  Rotina que realiza a criacao de novas situacoes de frequencia de acordo com informacoes da Etapa 7
     * 
     *  @param  $nreventomAdicionalNoturno,
     *  @param  $dtIniVigencia
     * 
     *  @return self::setAssocEventoSituFreq()
     * 
     */
    public function setSituacoesFrequenciaEventosEtapa7RPF(
            $nreventomAdicionalNoturno,
            $dtIniVigencia
    ) {
        
        return $this->setAssocEventoSituFreq(
            self::RPF_ASSOC_SF_ADICIONAL_NOTURNO['DSSITUFREQUENCIA'],
            self::RPF_ASSOC_SF_ADICIONAL_NOTURNO['DSABREVIACAO'],
            $dtIniVigencia,
            $nreventomAdicionalNoturno,
            self::RPF_ASSOC_SF_ADICIONAL_NOTURNO['NRCALCSITUFREQ']
        );
    }
    
    /**
     *  Rotina que realiza o cadastro de uma nova situacao de frequencia.
     *      + Caso $nrEventom seja informado, a situacao eh associada a um novo parametro (PTO_SITUFREQPARAM) com o evento;
     *      + Caso $nrCalculoSituFreq seja informado, uma associacao de frequencia eh criada (PTO_ASSOCSITUFREQ) para que a situacao criada sobreecreva a de org 0
     * 
     *  @param  $dsSituFrequencia
     *  @param  $dsAbreviacao
     *  @param  $dtIniVigencia
     *  @param  $nrEventom          = null
     *  @param  $nrCalculoSituFreq  = null
     * 
     *  @retun  $novaSituFreq               [PtoSitufrequencia]
     * 
     */
    public function setAssocEventoSituFreq(
        string  $dsSituFrequencia,
        string  $dsAbreviacao,
        string  $dtIniVigencia,
                $nrEventom          = null,
                $nrCalculoSituFreq  = null
    ) {
            
        $novaSituFreq = $this->setSituFrequencia(
            $dsSituFrequencia,
            $dsAbreviacao,
            $dtIniVigencia
        );
        
        if (!empty($nrEventom)) {
            $this->setSituFreqParam(
                $novaSituFreq->getNrsitufrequencia(),
                $nrEventom
            );
        }
        
        if (!empty($nrCalculoSituFreq)) {
            $this->setAssocSituFreq(
                $novaSituFreq->getNrsitufrequencia(),
                $nrCalculoSituFreq
            );
        }
        
        return $novaSituFreq;
        
    }
    
    /**
     *  Rotina que retorna um nome padrao para ocorrencias de banco, a partir da base self::DESC_BANCOHORASPROG_PREFIX e de um sufixo informado
     * 
     *  @param  $suffixName
     * 
     */
    public function getDescricaoOcorrencia(string $suffixName = null) {
        return empty($suffixName) ? self::DESC_BANCOHORASPROG_PREFIX : self::DESC_BANCOHORASPROG_PREFIX . ' ' . $suffixName;
    }
    
    /**
     *  Rotina que cria alteracoes de banco de horas (PTO_ALTERBANCOHORAS) relacionando os vinculos ao banco de horas informados
     * 
     *  @param  $nrBancohoras
     *  @param  $vinculos
     *  @param  $dtIniVigencia
     * 
     */
    public function criaAlterBancoHorasEtapaFinalRPF(
                $nrBancohoras,
        Array   $vinculos,
        string  $dtIniVigencia
    ) {
        
        foreach ($vinculos as $vinculo) {
            $alteracaoBancoHorasExistente = $this->entityManager->getRepository(Repositories::PTO_ALTEBANCOHORAS)->findOneBy(
                array(
                    'nrorg'         => $this->hcmParameters->getNrorgTrabalho(),
                    'nrvinculom'    => $vinculo['NRVINCULOM']
                )
            );
            
            if (!empty($alteracaoBancoHorasExistente)) {
                throw new \Exception(Messages::RPF_VINCULOS_PARAM_FOUND);
            }
            
            
            $this->setAlteBancoHoras(
                $nrBancohoras,
                $vinculo['NRVINCULOM'],
                $dtIniVigencia,
                null
            );
        }
        
        return true;
    }
    
    /**
     *  Rotinas de cadastro de informações da parametrização
     * 
     */
    
    public function setBancoHoras(
        $descricao,
        $dtInicio,
        $dtFim
    ) {
        
        $dtInicio   = $dtInicio instanceof \DateTime ? $dtInicio : DateUtil::getDataDeString($dtInicio, DateUtil::FORMATO_BRASILEIRO, true);
        $dtFim      = $dtFim instanceof \DateTime ? $dtFim : DateUtil::getDataDeString($dtFim, DateUtil::FORMATO_BRASILEIRO, true);
        
        $defaults = [
            'IDSITUFREQS'   => '18000,18420,18020',
        ];
        
        $bancoHoras = $this->doctrineEntityUtil->setNewEntity(
            Repositories::PTO_BANCOHORAS,
            [
                'NRBANCOHORAS'          => \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'PTO_BANCOHORAS', 1, 12, true),
                'DSBANCOHORAS'          => $descricao,
                'DTINICIOBANCOHORAS'    => $dtInicio,
                'DTFIMBANCOHORAS'       => $dtFim,
                'IDSITUFREQS'           => $defaults['IDSITUFREQS'],
            ],
            true,
            true,
            false
        );
        
        return $bancoHoras;
    }
    
    public function setBancoHorasPeriodo(
        $nrBancoHoras,
        $dtInicio,
        $dtFim
    ) {
        
        $dtInicio   = $dtInicio instanceof \DateTime ? $dtInicio : DateUtil::getDataDeString($dtInicio, DateUtil::FORMATO_BRASILEIRO, true);
        $dtFim      = $dtFim instanceof \DateTime ? $dtFim : DateUtil::getDataDeString($dtFim, DateUtil::FORMATO_BRASILEIRO, true);
        
        $bancoHorasPeriodo = $this->doctrineEntityUtil->setNewEntity(
            Repositories::PTO_BANCOHORASPERIODO,
            [
                'NRBANCOHORASPERIODO'   => \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'PTO_BANCOHORASPERIODO', 1, 12, true),
                'NRBANCOHORAS'          => $nrBancoHoras,
                'DTINICIOPERIODOBHORAS' => $dtInicio,
                'DTFIMPERIODOBHORAS'    => $dtFim,
            ],
            true,
            true,
            false
        );
        
        return $bancoHorasPeriodo;
    }
    
    public function setBancoHorasProg(
        $nrBancoHoras,
        $nrTipoProgOcorrencia,
        $dtInicio,
        $dtFim                  = null,
        $vrCoeficienteAcrescimo = null,
        $dsBancoHorasProg       = self::DESC_BANCOHORASPROG_PREFIX,
        $nrSituFrequencia       = null
    ) {
        
        $dtInicio   = $dtInicio instanceof \DateTime ? $dtInicio : DateUtil::getDataDeString($dtInicio, DateUtil::FORMATO_BRASILEIRO, true);
        $dtFim      = $dtFim instanceof \DateTime || is_null($dtFim) ? $dtFim : DateUtil::getDataDeString($dtFim, DateUtil::FORMATO_BRASILEIRO, true);
        
        $defaults = [
            'NRSITUFREQGATILHO'         => null,
            'IDDIASSEMANA'              => OcorrenciaFrequenciaService::IDDIASSEMANA_DEFAULT,
            'IDTIPODIACALENDARIO'       => null,
            'IDTIPOHORARIO'             => OcorrenciaFrequenciaService::IDTIPOHORARIO_NORMAL,
            'QTMINGATILHOPROGBHORAS'    => null,
            'QTMAXGATILHOPROGBHORAS'    => null,
            'QTMINIMAPROGBHORAS'        => null,
            'QTMAXIMAPROGBHORAS'        => null,
            'DSOBSERVACAO'              => null,
        ];
        
        $bancoHorasProg = $this->doctrineEntityUtil->setNewEntity(
            Repositories::PTO_BANCOHORASPROG,
            [
                'NRBANCOHORASPROG'          => \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'PTO_BANCOHORASPROG', 1, 12, true),
                'NRBANCOHORAS'              => $nrBancoHoras,
                'DSBANCOHORASPROG'          => $dsBancoHorasProg,
                'NRSITUFREQUENCIA'          => $nrSituFrequencia,
                'NRSITUFREQGATILHO'         => $defaults['NRSITUFREQGATILHO'],
                'DTINICIOPROGBHORAS'        => $dtInicio,
                'DTFIMPROGBHORAS'           => $dtFim,
                'IDDIASSEMANA'              => $defaults['IDDIASSEMANA'],
                'IDTIPODIACALENDARIO'       => $defaults['IDTIPODIACALENDARIO'], 
                'IDTIPOHORARIO'             => $defaults['IDTIPOHORARIO'],
                'QTMINGATILHOPROGBHORAS'    => $defaults['QTMINGATILHOPROGBHORAS'],
                'QTMAXGATILHOPROGBHORAS'    => $defaults['QTMAXGATILHOPROGBHORAS'],
                'VRCOEFICIENTEACRESCIMO'    => $vrCoeficienteAcrescimo,
                'QTMINIMAPROGBHORAS'        => $defaults['QTMINIMAPROGBHORAS'],
                'QTMAXIMAPROGBHORAS'        => $defaults['QTMAXIMAPROGBHORAS'],
                'DSOBSERVACAO'              => $defaults['DSOBSERVACAO'],
                'NRTIPOPROGOCORRENCIA'      => $nrTipoProgOcorrencia,
            ],
            true,
            true,
            false
        );
        
        return $bancoHorasProg;
    }
    
    public function setAlteBancoHoras(
        $nrBancoHoras,
        $nrVinculom,
        $dtInicio,
        $dtFim          = null
    ) {
        
        $dtInicio   = $dtInicio instanceof \DateTime ? $dtInicio : DateUtil::getDataDeString($dtInicio, DateUtil::FORMATO_BRASILEIRO, true);
        $dtFim      = $dtFim instanceof \DateTime || is_null($dtFim) ? $dtFim : DateUtil::getDataDeString($dtFim, DateUtil::FORMATO_BRASILEIRO, true);
        
        $alteBancoHoras = $this->doctrineEntityUtil->setNewEntity(
            Repositories::PTO_ALTEBANCOHORAS,
            [
                'NRALTEBANCOHORAS'      => \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'PTO_ALTEBANCOHORAS', 1, 12, true),
                'NRBANCOHORAS'          => $nrBancoHoras,
                'NRVINCULOM'            => $nrVinculom,
                'DTINICIOALTEBHORAS'    => $dtInicio,
                'DTFIMALTEBHORAS'       => $dtFim,
            ],
            true,
            true,
            false
        );
        
        return $alteBancoHoras;
    }
    
    public function setSituFrequencia(
        $dsSituFrequencia,
        $dsAbreviacao,
        $dtIniVigencia,
        $dtFimVigencia      = null
    ) {
        
        $dtIniVigencia  = $dtIniVigencia instanceof \DateTime ? $dtIniVigencia : DateUtil::getDataDeString($dtIniVigencia, DateUtil::FORMATO_BRASILEIRO, true);
        $dtFimVigencia  = $dtFimVigencia instanceof \DateTime || is_null($dtFimVigencia) ? $dtFimVigencia : DateUtil::getDataDeString($dtFimVigencia, DateUtil::FORMATO_BRASILEIRO, true);
        
        // Manipulacao de entidade que sera utilizada em outras entidades no processo. Criada como 'pending'
        $novaSituFreq = $this->doctrineEntityUtil->setNewEntity(
            Repositories::PTO_SITUFREQUENCIA,
            [
                'NRSITUFREQUENCIA'  => \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'PTO_SITUFREQUENCIA', 1, 12, true),
                'DSSITUFREQUENCIA'  => $dsSituFrequencia,
                'DSABREVIACAO'      => $dsAbreviacao,
                'DTINIVIGENCIA'     => $dtIniVigencia,
                'DTFIMVIGENCIA'     => $dtFimVigencia,
                'NRADVERTSITUFREQ'  => null,
            ],
            true,
            false,
            false,
            true
        );
        
        return $novaSituFreq;
    }
    
    public function setSituFreqParam(
        $nrSituFrequencia,
        $nrEventom
    ) {
        
        $defaults = [
            'IDGERAEVENTOMESINCID'  => 'N',
            'IDPERDEDSR'            => 'N',
            'IDPERDEDSRQTDIFZERO'   => 'N',
            'QTOCORREPERDEDSR'      => 0,
            'QTREFDIAPERDEDSR'      => 0,
            'IDTIPOQTDEREFERENCIA'  => 'HRMI',
            'IDTIPOSITUACAOFREQ'    => 'N',
        ];
        
        $novaSituFreqParam = $this->doctrineEntityUtil->setNewEntity(
            Repositories::PTO_SITUFREQPARAM,
            [
                'NRSITUFREQPARAM'       => \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'PTO_SITUFREQPARAM', 1, 12, true),
                'NREVENTOM'             => $nrEventom,
                'IDGERAEVENTOMESINCID'  => $defaults['IDGERAEVENTOMESINCID'],
                'IDPERDEDSR'            => $defaults['IDPERDEDSR'],
                'IDPERDEDSRQTDIFZERO'   => $defaults['IDPERDEDSRQTDIFZERO'],
                'QTOCORREPERDEDSR'      => $defaults['QTOCORREPERDEDSR'],
                'QTREFDIAPERDEDSR'      => $defaults['QTREFDIAPERDEDSR'],
                'IDTIPOQTDEREFERENCIA'  => $defaults['IDTIPOQTDEREFERENCIA'],
                'IDTIPOSITUACAOFREQ'    => $defaults['IDTIPOSITUACAOFREQ'],
                'NRSITUFREQUENCIA'      => $nrSituFrequencia,
            ],
            true,
            true,
            false
        );
        
        return $novaSituFreqParam;
    }
    
    public function setAssocSituFreq(
        $nrSituFrequencia,
        $nrCalculoSituFreq
    ) {
        
        $novaAssocSituFreq = $this->doctrineEntityUtil->setNewEntity(
            Repositories::PTO_ASSOCSITUFREQ,
            [
                'NRASSOCSITUFREQ'   => \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'PTO_ASSOCSITUFREQ', 1, 12, true),
                'NRSITUFREQUENCIA'  => $nrSituFrequencia,
                'NRCALCULOSITUFREQ' => $nrCalculoSituFreq,
            ],
            true,
            true,
            false
        );
        
        return $novaAssocSituFreq;
    }
} 